// Copyright (c) 2010, Lawrence Livermore National Security, LLC. Produced at
// the Lawrence Livermore National Laboratory. LLNL-CODE-443211. All Rights
// reserved. See file COPYRIGHT for details.
//
// This file is part of the MFEM library. For more information and source code
// availability see http://mfem.org.
//
// MFEM is free software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License (as published by the Free
// Software Foundation) version 2.1 dated February 1999.

#ifndef MFEM_HEXAHEDRON
#define MFEM_HEXAHEDRON

#include "../config/config.hpp"
#include "element.hpp"

namespace mfem
{

/// Data type hexahedron element
class Hexahedron : public Element
{
protected:
   int indices[8];

public:
   typedef Geometry::Constants<Geometry::CUBE> geom_t;

   Hexahedron() : Element(Geometry::CUBE) { }

   /// Constructs hexahedron by specifying the indices and the attribute.
   Hexahedron(const int *ind, int attr = 1);

   /// Constructs hexahedron by specifying the indices and the attribute.
   Hexahedron(int ind1, int ind2, int ind3, int ind4,
              int ind5, int ind6, int ind7, int ind8, int attr = 1);

   /// Return element's type
   Type GetType() const { return Element::HEXAHEDRON; }

   /// Returns the indices of the element's vertices.
   virtual void GetVertices(Array<int> &v) const;

   virtual int *GetVertices() { return indices; }

   virtual int GetNVertices() const { return 8; }

   virtual int GetNEdges() const { return 12; }

   virtual const int *GetEdgeVertices(int ei) const
   { return geom_t::Edges[ei]; }

   virtual int GetNFaces(int &nFaceVertices) const
   { nFaceVertices = 4; return 6; }

   virtual const int *GetFaceVertices(int fi) const
   { return geom_t::FaceVert[fi]; }

   virtual Element *Duplicate(Mesh *m) const
   { return new Hexahedron(indices, attribute); }

   virtual ~Hexahedron() { }
};

extern TriLinear3DFiniteElement HexahedronFE;

}

#endif
